@@ -18,6 +18,15 @@ def test_simulation_load_export():
     assert SIM == SIM2, "original and loaded simulations are not the same"
 
 
+@clear_tmp
+def test_simulation_load_export_yaml():
+
+    path = "tests/tmp/simulation.yaml"
+    SIM.to_file(path)
+    SIM2 = Simulation.from_file(path)
+    assert SIM == SIM2, "original and loaded simulations are not the same"
+
+
 @clear_tmp
 def test_simulation_preserve_types():
 
@@ -48,6 +57,7 @@ def test_simulation_preserve_types():
                 pol_angle=2.0,
             ),
             GaussianBeam(
+                center=(0, 0, -4),
                 size=(1, 1, 0),
                 source_time=st,
                 direction="+",
