@@ -54,7 +54,7 @@ def test_sim():
             FieldMonitor(size=(0, 0, 0), center=(0, 0, 0), freqs=[1, 2], name="point"),
             FluxTimeMonitor(size=(1, 1, 0), center=(0, 0, 0), interval=10, name="plane"),
         ],
-        symmetry=(0, 0, 0),
+        symmetry=(0, 1, -1),
         pml_layers=(
             PML(num_layers=20),
             StablePML(num_layers=30),
@@ -65,6 +65,23 @@ def test_sim():
         subpixel=False,
     )
 
+    dt = sim.dt
+    tm = sim.tmesh
+    sim.validate_pre_upload()
+    ms = sim.mediums
+    mm = sim.medium_map
+    m = sim.get_monitor_by_name("point")
+    s = sim.background_structure
+    sim.plot(x=0)
+    sim.plot_eps(x=0)
+    sim.num_pml_layers
+    sim.plot_grid(x=0)
+    sim.frequency_range
+    sim.grid
+    sim.num_cells
+    sim.discretize(m)
+    sim.epsilon(m)
+
 
 def _test_version():
     """ensure there's a version in simulation"""
@@ -293,13 +310,13 @@ def test_num_mediums():
         structures.append(
             Structure(geometry=Box(size=(1, 1, 1)), medium=Medium(permittivity=i + 1))
         )
-    sim = Simulation(size=(1, 1, 1), grid_spec=grid_spec, structures=structures, run_time=1e-12)
+    sim = Simulation(size=(5, 5, 5), grid_spec=grid_spec, structures=structures, run_time=1e-12)
 
     with pytest.raises(SetupError):
         structures.append(
             Structure(geometry=Box(size=(1, 1, 1)), medium=Medium(permittivity=i + 2))
         )
-        sim = Simulation(size=(1, 1, 1), grid_spec=grid_spec, structures=structures, run_time=1e-12)
+        sim = Simulation(size=(5, 5, 5), grid_spec=grid_spec, structures=structures, run_time=1e-12)
 
 
 """ geometry """
@@ -676,17 +693,21 @@ def test_source_times():
     g = GaussianPulse(freq0=1, fwidth=0.1)
     ts = np.linspace(0, 30, 1001)
     g.amp_time(ts)
+    g.plot(ts)
 
-    # test we can make cq pulse
-    # c = CW(freq0=1, fwidth=0.1)
-    # ts = np.linspace(0, 30, 1001)
-    # c.amp_time(ts)
+    # test we can make cw pulse
+    from tidy3d.components.source import ContinuousWave
+
+    c = ContinuousWave(freq0=1, fwidth=0.1)
+    ts = np.linspace(0, 30, 1001)
+    c.amp_time(ts)
 
 
 def test_dipole():
 
     g = GaussianPulse(freq0=1, fwidth=0.1)
     p = PointDipole(center=(1, 2, 3), source_time=g, polarization="Ex")
+    p.plot(y=2)
 
     with pytest.raises(pydantic.ValidationError) as e_info:
         p = PointDipole(size=(1, 1, 1), source_time=g, center=(1, 2, 3), polarization="Ex")
@@ -698,21 +719,48 @@ def test_FieldSource():
 
     # test we can make planewave
     s = PlaneWave(size=(0, inf, inf), source_time=g, pol_angle=np.pi / 2, direction="+")
+    s.plot(y=0)
 
     # test we can make gaussian beam
     s = GaussianBeam(size=(0, 1, 1), source_time=g, pol_angle=np.pi / 2, direction="+")
+    s.plot(y=0)
+
+    # test we can make an astigmatic gaussian beam
+    s = AstigmaticGaussianBeam(
+        size=(0, 1, 1),
+        source_time=g,
+        pol_angle=np.pi / 2,
+        direction="+",
+        waist_sizes=(0.2, 0.4),
+        waist_distances=(0.1, 0.3),
+    )
 
     # test we can make mode source
     s = ModeSource(size=(0, 1, 1), direction="+", source_time=g, mode_spec=mode_spec, mode_index=0)
+    s.plot(y=0)
 
-    # test that non-planar geometry crashes plane wave and gaussian beam
+    # test that non-planar geometry crashes plane wave and gaussian beams
     with pytest.raises(ValidationError) as e_info:
         s = PlaneWave(size=(1, 1, 1), source_time=g, pol_angle=np.pi / 2, direction="+")
     with pytest.raises(ValidationError) as e_info:
         s = GaussianBeam(size=(1, 1, 1), source_time=g, pol_angle=np.pi / 2, direction="+")
+    with pytest.raises(ValidationError) as e_info:
+        s = AstigmaticGaussianBeam(
+            size=(1, 1, 1),
+            source_time=g,
+            pol_angle=np.pi / 2,
+            direction="+",
+            waist_sizes=(0.2, 0.4),
+            waist_distances=(0.1, 0.3),
+        )
     with pytest.raises(ValidationError) as e_info:
         s = ModeSource(size=(1, 1, 1), source_time=g, mode_spec=mode_spec)
 
+    from tidy3d.components.source import TFSF
+
+    s = TFSF(size=(1, 1, 1), direction="+", source_time=g, injection_axis=2)
+    s.plot(z=0)
+
 
 """ monitors """
 
@@ -722,7 +770,28 @@ def test_monitor():
     size = (1, 2, 3)
     center = (1, 2, 3)
 
-    m = FieldMonitor(size=size, center=center, freqs=[1, 2, 3], name="test_monitor")
+    m1 = FieldMonitor(size=size, center=center, freqs=[1, 2, 3], name="test_monitor")
+    m1.surfaces()
+    m2 = FieldTimeMonitor(size=size, center=center, name="test_mon")
+    m3 = FluxMonitor(size=(1, 1, 0), center=center, freqs=[1, 2, 3], name="test_mon")
+    m4 = FluxTimeMonitor(size=(1, 1, 0), center=center, name="test_mon")
+    m5 = ModeMonitor(
+        size=(1, 1, 0), center=center, mode_spec=ModeSpec(), freqs=[1, 2, 3], name="test_mon"
+    )
+    m6 = ModeFieldMonitor(
+        size=(1, 1, 0), center=center, mode_spec=ModeSpec(), freqs=[1, 2, 3], name="test_mon"
+    )
+    m7 = PermittivityMonitor(size=size, center=center, freqs=[1, 2, 3], name="perm")
+
+    tmesh = np.linspace(0, 1, 10)
+
+    for m in [m1, m2, m3, m4, m5, m6, m7]:
+        m.plot(y=2)
+        m.storage_size(num_cells=100, tmesh=tmesh)
+
+    for m in [m2, m4]:
+        m.time_inds(tmesh=tmesh)
+        m.num_steps(tmesh=tmesh)
 
 
 def test_monitor_plane():
