@@ -327,16 +327,16 @@ class SpatialCollectionData(CollectionData, ABC):
 
     """ Attributes storing details about any symmetries that can be used to expand the data. """
 
-    symmetry_center: Coordinate = pd.Field(
-        None, title="Symmetry Center", description="Position of the symmetry planes in x, y, and z."
-    )
-
     symmetry: Tuple[Symmetry, Symmetry, Symmetry] = pd.Field(
         (0, 0, 0),
         title="Symmetry Eigenvalues",
         description="igenvalues of the symmetry under reflection in x, y, and z.",
     )
 
+    symmetry_center: Coordinate = pd.Field(
+        None, title="Symmetry Center", description="Position of the symmetry planes in x, y, and z."
+    )
+
     expanded_grid: Dict[str, Coords] = pd.Field(
         {},
         title="Expanded Grid",
@@ -357,6 +357,13 @@ class SpatialCollectionData(CollectionData, ABC):
         "the corresponding ``_sym_dict`` value times the ``self.symmetry`` eigenvalue is used.",
     """
 
+    @pd.validator("symmetry_center", always=True)
+    def _defined_if_sym_present(cls, val, values):
+        """If symmetry required, must have symmetry_center."""
+        if any(sym != 0 for sym in values.get("symmetry")):
+            assert val is not None, "symmetry_center must be supplied."
+        return val
+
     def colocate(self, x, y, z) -> xr.Dataset:
         """colocate all of the data at a set of x, y, z coordinates.
 
@@ -385,6 +392,7 @@ class SpatialCollectionData(CollectionData, ABC):
         centered_data_dict = {}
         for field_name, field_data in self.data_dict.items():
             centered_data_array = field_data.data
+            interp_dict = {}
             for coord_name in "xyz":
                 if len(centered_data_array.coords[coord_name]) <= 1:
                     # centered_data_array = centered_data_array.isel(**{coord_name:0})
@@ -393,9 +401,8 @@ class SpatialCollectionData(CollectionData, ABC):
                     centered_data_array = centered_data_array.assign_coords(**coord_kwargs)
                     centered_data_array = centered_data_array.isel(**{coord_name: 0})
                 else:
-                    coord_vals = coord_val_map[coord_name]
-                    centered_data_array = centered_data_array.interp(**{coord_name: coord_vals})
-            centered_data_dict[field_name] = centered_data_array
+                    interp_dict[coord_name] = coord_val_map[coord_name]
+            centered_data_dict[field_name] = centered_data_array.interp(**interp_dict)
         return xr.Dataset(centered_data_dict)
 
     @property
@@ -434,8 +441,8 @@ class SpatialCollectionData(CollectionData, ABC):
                 coords_interp[flip_inds] = 2 * center - coords[flip_inds]
 
                 # Interpolate. There generally shouldn't be values out of bounds except potentially
-                # when handling modes, in which case we set such values to zero.
-                new_data = new_data.interp({dim_name: coords_interp}, kwargs={"fill_value": 0.0})
+                # when handling modes, in which case they should be at the boundary and close to 0.
+                new_data = new_data.sel({dim_name: coords_interp}, method="nearest")
                 new_data = new_data.assign_coords({dim_name: coords})
 
                 sym_eval = self._sym_dict.get(data_key)
@@ -686,8 +693,9 @@ class ModeAmpsData(AbstractModeData):
     Example
     -------
     >>> f = np.linspace(2e14, 3e14, 1001)
-    >>> values = (1+1j) * np.random.random((1, 2, len(f)))
-    >>> data = ModeAmpsData(values=values, direction=['+'], mode_index=np.arange(1, 3), f=f)
+    >>> mode_index = np.arange(1, 3)
+    >>> values = (1+1j) * np.random.random((2, len(f), len(mode_index)))
+    >>> data = ModeAmpsData(values=values, direction=['+', '-'], mode_index=mode_index, f=f)
     """
 
     direction: List[Direction] = pd.Field(
@@ -721,7 +729,7 @@ class ModeIndexData(AbstractModeData):
     Example
     -------
     >>> f = np.linspace(2e14, 3e14, 1001)
-    >>> values = (1+1j) * np.random.random((2, len(f)))
+    >>> values = (1+1j) * np.random.random((len(f), 2))
     >>> data = ModeIndexData(values=values, mode_index=np.arange(1, 3), f=f)
     """
 
@@ -922,10 +930,10 @@ class ModeData(CollectionData):
     -------
     >>> f = np.linspace(1e14, 2e14, 1001)
     >>> mode_index = np.arange(2)
+    >>> amps = (1+1j) * np.random.random((2, len(f), len(mode_index)))
+    >>> amps_data = ModeAmpsData(values=amps, f=f, mode_index=mode_index)
     >>> n_complex = (1+1j) * np.random.random((len(f), len(mode_index)))
     >>> index_data = ModeIndexData(values=n_complex, f=f, mode_index=mode_index)
-    >>> amps = (1+1j) * np.random.random((2, len(f), len(mode_index)))
-    >>> amps_data = ModeIndexData(values=amps, f=f, mode_index=mode_index)
     >>> data = ModeData(data_dict={'n_complex': index_data, 'amps': amps_data})
     """
 
@@ -970,7 +978,20 @@ class ModeData(CollectionData):
 
 
 class ModeFieldData(AbstractFieldData):
-    """Like FieldData but with extra dimension ``mode_index``."""
+    """Like FieldData but with extra dimension ``mode_index``.
+
+
+    Example
+    -------
+    >>> f = np.linspace(1e14, 2e14, 1001)
+    >>> x = np.linspace(-1, 1, 10)
+    >>> y = np.linspace(-2, 2, 20)
+    >>> z = np.linspace(0, 0, 1)
+    >>> mode_index = np.arange(0, 4)
+    >>> values = (1+1j) * np.random.random((len(x), len(y), len(z), len(f), len(mode_index)))
+    >>> field = ScalarModeFieldData(values=values, x=x, y=y, z=z, f=f, mode_index=mode_index)
+    >>> data = ModeFieldData(data_dict={'Ex': field, 'Ey': field})
+    """
 
     data_dict: Dict[str, ScalarModeFieldData] = pd.Field(
         ...,
@@ -1033,6 +1054,8 @@ class AbstractSimulationData(Tidy3dBaseModel, ABC):
         freq: float = None,
         eps_alpha: float = 0.2,
         robust: bool = True,
+        vmin: float = None,
+        vmax: float = None,
         ax: Ax = None,
     ) -> Ax:
         """Plot the field data for a monitor with simulation plot overlayed.
@@ -1054,8 +1077,15 @@ class AbstractSimulationData(Tidy3dBaseModel, ABC):
             Opacity of the structure permittivity.
             Must be between 0 and 1 (inclusive).
         robust : bool = True
-            If specified, uses the 2nd and 98th percentiles of the data to compute the color limits.
-            This helps in visualizing the field patterns especially in the presence of a source.
+            If True and vmin or vmax are absent, uses the 2nd and 98th percentiles of the data
+            to compute the color limits. This helps in visualizing the field patterns especially
+            in the presence of a source.
+        vmin : float = None
+            The lower bound of data range that the colormap covers. If `None`, they are
+            inferred from the data and other keyword arguments.
+        vmax : float = None
+            The upper bound of data range that the colormap covers. If `None`, they are
+            inferred from the data and other keyword arguments.
         ax : matplotlib.axes._subplots.Axes = None
             matplotlib axes to plot on, if not specified, one is created.
 
@@ -1098,7 +1128,9 @@ class AbstractSimulationData(Tidy3dBaseModel, ABC):
         xy_coord_labels = list("xyz")
         xy_coord_labels.pop(axis)
         x_coord_label, y_coord_label = xy_coord_labels  # pylint:disable=unbalanced-tuple-unpacking
-        field_data.plot(ax=ax, x=x_coord_label, y=y_coord_label, robust=robust, cmap=cmap)
+        field_data.plot(
+            ax=ax, x=x_coord_label, y=y_coord_label, robust=robust, cmap=cmap, vmin=vmin, vmax=vmax
+        )
 
         # plot the simulation epsilon
         ax = self.simulation.plot_structures_eps(
@@ -1252,6 +1284,8 @@ class SimulationData(AbstractSimulationData):
         mode_index: int = None,
         eps_alpha: float = 0.2,
         robust: bool = True,
+        vmin: float = None,
+        vmax: float = None,
         ax: Ax = None,
     ) -> Ax:
         """Plot the field data for a monitor with simulation plot overlayed.
@@ -1284,8 +1318,15 @@ class SimulationData(AbstractSimulationData):
             Opacity of the structure permittivity.
             Must be between 0 and 1 (inclusive).
         robust : bool = True
-            If specified, uses the 2nd and 98th percentiles of the data to compute the color limits.
-            This helps in visualizing the field patterns especially in the presence of a source.
+            If True and vmin or vmax are absent, uses the 2nd and 98th percentiles of the data
+            to compute the color limits. This helps in visualizing the field patterns especially
+            in the presence of a source.
+        vmin : float = None
+            The lower bound of data range that the colormap covers. If `None`, they are
+            inferred from the data and other keyword arguments.
+        vmax : float = None
+            The upper bound of data range that the colormap covers. If `None`, they are
+            inferred from the data and other keyword arguments.
         ax : matplotlib.axes._subplots.Axes = None
             matplotlib axes to plot on, if not specified, one is created.
 
@@ -1350,6 +1391,8 @@ class SimulationData(AbstractSimulationData):
             freq=freq,
             eps_alpha=eps_alpha,
             robust=robust,
+            vmin=vmin,
+            vmax=vmax,
             ax=ax,
         )
 
