@@ -3,10 +3,11 @@
 
 from abc import ABC, abstractmethod
 from typing import List, Tuple, Union, Any, Callable
+from math import isclose
 
 import pydantic
 import numpy as np
-
+from gdspy import boolean
 from shapely.geometry import Point, Polygon, box, MultiPolygon
 from descartes import PolygonPatch
 
@@ -22,6 +23,7 @@ from ..constants import MICROMETER, LARGE_NUMBER, RADIAN
 # for sampling polygon in slanted polyslab along  z-direction for
 # validating polygon to be non_intersecting.
 _N_SAMPLE_POLYGON_INTERSECT = 100
+_IS_CLOSE_RTOL = np.finfo(float).eps
 
 
 class Geometry(Tidy3dBaseModel, ABC):
@@ -473,6 +475,9 @@ class Geometry(Tidy3dBaseModel, ABC):
             Angle of rotation counter-clockwise around the axis (rad).
         """
 
+        if isclose(angle % (2 * np.pi), 0):
+            return points
+
         # Normalized axis vector components
         (ux, uy, uz) = axis / np.linalg.norm(axis)
 
@@ -493,7 +498,7 @@ class Geometry(Tidy3dBaseModel, ABC):
         if len(points.shape) == 1:
             return rot_mat @ points
 
-        return np.einsum("ij,jp...->ip...", rot_mat, points)
+        return np.tensordot(rot_mat, points, axes=1)
 
     def reflect_points(
         self,
@@ -1078,6 +1083,7 @@ class Cylinder(Circular, Planar):
             Whether point ``(x,y,z)`` is inside geometry.
         """
         z0, (x0, y0) = self.pop_axis(self.center, axis=self.axis)
+        z, (x, y) = self.pop_axis((x, y, z), axis=self.axis)
         dist_x = np.abs(x - x0)
         dist_y = np.abs(y - y0)
         dist_z = np.abs(z - z0)
@@ -1164,9 +1170,8 @@ class PolySlab(Planar):
 
     @pydantic.validator("vertices", always=True)
     def correct_shape(cls, val):
-        """makes sure vertices size is correct. Remove duplicate
-        neighbouring vertices, orient vertices in CCW direction.
-        make sure no intersecting edges.
+        """Makes sure vertices size is correct.
+        Make sure no intersecting edges.
         """
 
         val_np = PolySlab.vertices_to_array(val)
@@ -1179,20 +1184,13 @@ class PolySlab(Planar):
                 f"Given array with shape of {shape}."
             )
 
-        # remove duplicate neighbouring vertices, and orient in CCW direction
-        vertices = PolySlab._proper_vertices(val)
-        if vertices.shape[0] < 3:
-            raise SetupError(
-                "At least 3 vertices (after removing duplicate neighbouring vertices) "
-                "need to be supplied to setup the polygon."
-            )
-
-        if not Polygon(vertices).is_valid:
+        # make sure no self-intersecting edges
+        if not Polygon(val_np).is_valid:
             raise SetupError(
                 "A valid Polygon may not possess any intersecting or overlapping edges."
             )
 
-        return PolySlab.array_to_vertices(val_np)
+        return val
 
     @pydantic.validator("vertices", always=True)
     def set_center(cls, val, values):
@@ -1224,44 +1222,53 @@ class PolySlab(Planar):
         if they are self-intersecting.
         """
 
+        # no need to valiate anything here
+        if isclose(values["dilation"], 0) and isclose(values["sidewall_angle"], 0):
+            return val
+
         ## First, make sure no vertex-vertex crossing in the base
         ## 1) obviously, vertex-vertex crossing can occur during erosion
         ## 2) for concave polygon, the crossing can even occur during dilation
         val_np = PolySlab._proper_vertices(val)
-        base = PolySlab._shift_vertices(val_np, values["dilation"])[0]
+        if isclose(values["dilation"], 0):
+            # no need to validate the base for 0 dilation
+            base = val_np
+        else:
+            base = PolySlab._shift_vertices(val_np, values["dilation"])[0]
 
-        # compute distance between vertices after dilation to detect vertex-vertex
-        # crossing events
-        cross_val, max_dist = PolySlab._crossing_detection(val_np, values["dilation"])
-        if cross_val:
-            # 1) crossing during erosion
-            if values["dilation"] < 0:
-                # too much erosion
+            # compute distance between vertices after dilation to detect vertex-vertex
+            # crossing events
+            cross_val, max_dist = PolySlab._crossing_detection(val_np, values["dilation"])
+            if cross_val:
+                # 1) crossing during erosion
+                if values["dilation"] < 0:
+                    # too much erosion
+                    raise SetupError(
+                        "Erosion value (-dilation) is too large. Some edges in the base polygon "
+                        f"are fully eroded. Maximal erosion should be {max_dist:.3e} "
+                        "for this polygon. Support for vertices crossing under "
+                        "significant erosion will be available in future releases."
+                    )
+                # 2) crossing during dilation in concave polygon
                 raise SetupError(
-                    "Erosion value (-dilation) is too large. Some edges in the base polygon "
-                    f"are fully eroded. Maximal erosion should be {max_dist:.3e} "
-                    "for this polygon. Support for vertices crossing under "
-                    "significant erosion will be available in future releases."
+                    "Dilation value is too large in a concave polygon, resulting in "
+                    "vertices crossing. "
+                    f"Maximal dilation should be {max_dist:.3e} for this polygon. "
+                    "Support for vertices crossing under significant dilation "
+                    "in concave polygons will be available in future releases."
+                )
+            # If no vertex-vertex crossing is detected, but the polygon is still self-intersecting.
+            # it is attributed to vertex-edge crossing.
+            if not Polygon(base).is_valid:
+                raise SetupError(
+                    "Dilation/Erosion value is too large, resulting in "
+                    "vertex-edge crossing, and thus self-intersecting polygons. "
+                    "Support for self-intersecting polygons under significant dilation "
+                    "will be available in future releases."
                 )
-            # 2) crossing during dilation in concave polygon
-            raise SetupError(
-                "Dilation value is too large in a concave polygon, resulting in "
-                "vertices crossing. "
-                f"Maximal dilation should be {max_dist:.3e} for this polygon. "
-                "Support for vertices crossing under significant dilation "
-                "in concave polygons will be available in future releases."
-            )
-        # If no vertex-vertex crossing is detected, but the polygon is still self-intersecting.
-        # it is attributed to vertex-edge crossing.
-        if not Polygon(base).is_valid:
-            raise SetupError(
-                "Dilation/Erosion value is too large, resulting in "
-                "vertex-edge crossing, and thus self-intersecting polygons. "
-                "Support for self-intersecting polygons under significant dilation "
-                "will be available in future releases."
-            )
 
-        if np.isclose(values["sidewall_angle"], 0):
+        # Second, validate slanted wall case
+        if isclose(values["sidewall_angle"], 0):
             return val
 
         # For Slanted PolySlab. Similar procedure to validate
@@ -1364,6 +1371,8 @@ class PolySlab(Planar):
         # apply scaling and convert vertices into polyslabs
         all_vertices = [vertices * gds_scale for vertices in all_vertices]
         all_vertices = [vertices.tolist() for vertices in all_vertices]
+        poly_set = boolean(all_vertices, None, "or")
+
         return [
             cls(
                 vertices=verts,
@@ -1373,7 +1382,7 @@ class PolySlab(Planar):
                 sidewall_angle=sidewall_angle,
                 **kwargs,
             )
-            for verts in all_vertices
+            for verts in poly_set.polygons
         ]
 
     @property
@@ -1386,6 +1395,7 @@ class PolySlab(Planar):
     @property
     def base_polygon(self) -> tidynumpy:
         """The polygon at the base after potential dilation operation.
+        The vertices will always be transformed to be "proper".
 
         Returns
         -------
@@ -1408,6 +1418,21 @@ class PolySlab(Planar):
         dist = -self.length * self._tanq
         return self._shift_vertices(self.base_polygon, dist)[0]
 
+    @property
+    def _base_polygon(self) -> tidynumpy:
+        """Similar as `base_polygon`, but simply return self.vertices
+        in the absence of dilation operation.
+
+        Returns
+        -------
+        tidynumpy
+            The vertices of the polygon at the base.
+        """
+        if isclose(self.sidewall_angle, 0) and isclose(self.dilation, 0):
+            return PolySlab.vertices_to_array(self.vertices)
+
+        return self.base_polygon
+
     def inside(self, x, y, z) -> bool:  # pylint:disable=too-many-locals
         """Returns True if point ``(x,y,z)`` inside volume of geometry.
         For slanted polyslab and x/y/z to be np.ndarray, a loop over z-axis
@@ -1453,8 +1478,8 @@ class PolySlab(Planar):
             ys_slab = y[inside_height]
 
             # vertical sidewall
-            if np.isclose(self.sidewall_angle, 0):
-                face_polygon = Polygon(self.base_polygon)
+            if isclose(self.sidewall_angle, 0):
+                face_polygon = Polygon(self._base_polygon)
                 fun_contain = contains_pointwise(face_polygon)
                 contains_vectorized = np.vectorize(fun_contain, signature="(n)->()")
                 points_stacked = np.stack((xs_slab, ys_slab), axis=1)
@@ -1473,7 +1498,7 @@ class PolySlab(Planar):
                     inside_polygon_slab = contains_vectorized(points_stacked)
                     inside_polygon[:, :, z_i] = inside_polygon_slab.reshape(x.shape[:2])
         else:
-            vertices_z = self._shift_vertices(self.base_polygon, dist)[0]
+            vertices_z = self._shift_vertices(self._base_polygon, dist)[0]
             face_polygon = Polygon(vertices_z)
             point = Point(x, y)
             inside_polygon = face_polygon.covers(point)
@@ -1497,7 +1522,7 @@ class PolySlab(Planar):
         z0, _ = self.pop_axis(self.center, axis=self.axis)
         z_local = z - z0 + self.length / 2  # distance to the base
         dist = -z_local * self._tanq
-        vertices_z = self._shift_vertices(self.base_polygon, dist)[0]
+        vertices_z = self._shift_vertices(self._base_polygon, dist)[0]
         return [Polygon(vertices_z)]
 
     def _intersections_side(self, position, axis) -> list:  # pylint:disable=too-many-locals
@@ -1552,10 +1577,10 @@ class PolySlab(Planar):
 
             # vertices for the base of each subsection
             dist = -h_base * self._tanq
-            vertices = self._shift_vertices(self.base_polygon, dist)[0]
+            vertices = self._shift_vertices(self._base_polygon, dist)[0]
 
             # for vertical sidewall, no need for complications
-            if np.isclose(self.sidewall_angle, 0):
+            if isclose(self.sidewall_angle, 0):
                 ints_y, ints_angle = self._find_intersecting_ys_angle_vertical(
                     vertices, position, axis
                 )
@@ -1571,7 +1596,7 @@ class PolySlab(Planar):
                 minx, miny = self._order_by_axis(plane_val=y_min, axis_val=z_min, axis=axis)
                 maxx, maxy = self._order_by_axis(plane_val=y_max, axis_val=z_max, axis=axis)
 
-                if np.isclose(self.sidewall_angle, 0):
+                if isclose(self.sidewall_angle, 0):
                     polys.append(box(minx=minx, miny=miny, maxx=maxx, maxy=maxy))
                 else:
                     angle_min = ints_angle[2 * y_index]
@@ -1627,7 +1652,7 @@ class PolySlab(Planar):
         np.ndarray
             Height (relative to the base) where the plane will intersect with vertices.
         """
-        if np.isclose(self.sidewall_angle, 0):
+        if isclose(self.sidewall_angle, 0):
             return np.array([])
 
         vertices = self.base_polygon.copy()
@@ -1636,7 +1661,7 @@ class PolySlab(Planar):
         dist = 1.0
         shift_x, shift_y = PolySlab._shift_vertices(vertices, dist)[2]
         shift_val = shift_x if axis == 0 else shift_y
-        shift_val[np.isclose(shift_val, 0)] = np.inf  # for static vertices
+        shift_val[np.isclose(shift_val, 0, rtol=_IS_CLOSE_RTOL)] = np.inf  # for static vertices
 
         # distance to the plane in the direction of vertex shifting
         distance = vertices[:, axis] - position
@@ -1767,7 +1792,7 @@ class PolySlab(Planar):
 
         ## Second part, plane intersects directly with vertices
         # vertices on the intersection
-        intersects_on = np.isclose(vertices_axis[:, 0], position)
+        intersects_on = np.isclose(vertices_axis[:, 0], position, rtol=_IS_CLOSE_RTOL)
         iverts_on = vertices_axis[intersects_on]
         # position of the neighbouring vertices
         iverts_b = vertices_b[intersects_on]
@@ -1788,7 +1813,7 @@ class PolySlab(Planar):
             if (x_b - position) * (x_f - position) < 0:
                 ints_y.append(y_on)
                 # vertices keep on the plane
-                if np.isclose(shift_local, 0):
+                if np.isclose(shift_local, 0, rtol=_IS_CLOSE_RTOL):
                     ints_angle.append(np.pi / 2)
                 else:
                     x1, y1 = x_b, y_b
@@ -1834,20 +1859,38 @@ class PolySlab(Planar):
         return ints_y_sort, ints_angle_sort
 
     @property
-    def _bounds(self):
+    def bounds(self) -> Bound:
+        """Returns bounding box min and max coordinates. The dilation and slant angle are not
+        taken into account exactly for speed. Instead, the polygon may be slightly smaller than
+        the returned bounds, but it should always be fully contained.
 
-        # get the min and max points in polygon plane
-        xpoints_base = tuple(c[0] for c in self.base_polygon)
-        ypoints_base = tuple(c[1] for c in self.base_polygon)
-        xpoints_top = tuple(c[0] for c in self.top_polygon)
-        ypoints_top = tuple(c[1] for c in self.top_polygon)
-        xmin = min(min(xpoints_base), min(xpoints_top))
-        ymin = min(min(ypoints_base), min(ypoints_top))
-        xmax = max(max(xpoints_base), max(xpoints_top))
-        ymax = max(max(ypoints_base), max(ypoints_top))
+        Returns
+        -------
+        Tuple[float, float, float], Tuple[float, float float]
+            Min and max bounds packaged as ``(minx, miny, minz), (maxx, maxy, maxz)``.
+        """
+
+        # check for the maximum possible contribution from dilation/slant on each side
+        max_offset = self.dilation + max(0, -self._tanq * self.length)
+
+        # special care when dilated
+        if max_offset > 0:
+            dilated_vertices = self._shift_vertices(
+                self._proper_vertices(self.vertices), max_offset
+            )[0]
+            xmin, ymin = np.amin(dilated_vertices, axis=0)
+            xmax, ymax = np.amax(dilated_vertices, axis=0)
+        else:
+            # otherwise, bounds are directly based on the supplied vertices
+            xmin, ymin = np.amin(self.vertices, axis=0)
+            xmax, ymax = np.amax(self.vertices, axis=0)
+
+        # get bounds in (local) z
         z0, _ = self.pop_axis(self.center, axis=self.axis)
         zmin = z0 - self.length / 2
         zmax = z0 + self.length / 2
+
+        # rearrange axes
         coords_min = self.unpop_axis(zmin, (xmin, ymin), axis=self.axis)
         coords_max = self.unpop_axis(zmax, (xmax, ymax), axis=self.axis)
         return (tuple(coords_min), tuple(coords_max))
@@ -1908,7 +1951,7 @@ class PolySlab(Planar):
 
         vertices_f = np.roll(vertices.copy(), shift=-1, axis=0)
         vertices_diff = np.linalg.norm(vertices - vertices_f, axis=1)
-        return vertices[~np.isclose(vertices_diff, 0)]
+        return vertices[~np.isclose(vertices_diff, 0, rtol=_IS_CLOSE_RTOL)]
 
     @staticmethod
     def _crossing_detection(vertices: np.ndarray, dist: float) -> Tuple[bool, float]:
@@ -1993,7 +2036,7 @@ class PolySlab(Planar):
             Shift along x and y direction.
         """
 
-        if np.isclose(dist, 0):
+        if isclose(dist, 0):
             return vertices, np.zeros(vertices.shape[0], dtype=float), None
 
         def rot90(v):
@@ -2022,7 +2065,9 @@ class PolySlab(Planar):
         det = cross(asm, asp)
 
         tan_half_angle = np.where(
-            np.isclose(det, 0), 0.0, cross(asm, rot90(asm - asp)) / (det + np.isclose(det, 0))
+            np.isclose(det, 0, rtol=_IS_CLOSE_RTOL),
+            0.0,
+            cross(asm, rot90(asm - asp)) / (det + np.isclose(det, 0, rtol=_IS_CLOSE_RTOL)),
         )
         parallel_shift = dist * tan_half_angle
 
