@@ -1,11 +1,15 @@
-""" Collection of functions for automatically generating a nonuniform grid. """
+# pylint:disable=too-many-lines
+"""Collection of functions for automatically generating a nonuniform grid. """
 
 from abc import ABC, abstractmethod
-from typing import Tuple, List, Union
+from typing import Tuple, List, Union, Dict
+from math import isclose
 
 import pydantic as pd
 import numpy as np
 from scipy.optimize import root_scalar
+from shapely.strtree import STRtree
+from shapely.geometry import Polygon
 
 from ..base import Tidy3dBaseModel
 from ..types import Axis, Array
@@ -14,6 +18,18 @@ from ...log import SetupError, ValidationError
 from ...constants import C_0, fp_eps
 
 
+# pylint:disable=abstract-method
+class IndexedBox(Polygon):
+    """A 2D shapely box with an extra ``str_ind`` variable to keep structure indexes."""
+
+    def __init__(self, minx, miny, maxx, maxy):
+        """Initialize with the same arguments as ``shapely.geometry.box``."""
+        coords = [(maxx, miny), (maxx, maxy), (minx, maxy), (minx, miny)]
+        super().__init__(coords)
+
+        self.str_ind = None
+
+
 class Mesher(Tidy3dBaseModel, ABC):
     """Abstract class for automatic meshing."""
 
@@ -42,7 +58,7 @@ class GradedMesher(Mesher):
     """Implements automatic nonuniform meshing with a set minimum steps per wavelength and
     a graded mesh expanding from higher- to lower-resolution regions."""
 
-    # pylint:disable=too-many-statements,too-many-locals
+    # pylint:disable=too-many-locals
     def parse_structures(
         self,
         axis: Axis,
@@ -86,6 +102,8 @@ class GradedMesher(Mesher):
 
         # Required maximum steps in every structure
         structure_steps = self.structure_steps(structures, wavelength, min_steps_per_wvl)
+        # Smallest of the maximum steps
+        min_step = np.amin(structure_steps)
 
         # If empty simulation, return
         if len(structures) == 1:
@@ -93,87 +111,149 @@ class GradedMesher(Mesher):
 
         # Bounding boxes with the meshing axis rotated to z
         struct_bbox = self.rotate_structure_bounds(structures, axis)
+        # Rtree from the 2D part of the bounding boxes
+        tree = self.bounds_2d_tree(struct_bbox)
 
+        intervals = {}
         # Array of coordinates of all intervals; add the simulation domain bounds already
-        interval_coords = np.array(domain_bounds)
+        intervals["coords"] = list(domain_bounds)
         # List of indexes of structures which are present in each interval
-        interval_structs = [[0]]  # will have len equal to len(interval_coords) - 1
-        # List of indexes of structures that every structre contains in 2D
-        struct_contains = [[]]  # will have len equal to len(structures)
+        intervals["structs"] = [[]]  # will have len equal to len(intervals["coords"]) - 1
 
-        for struct_ind in range(1, len(structures)):
-            bbox = struct_bbox[struct_ind]
+        # Iterate in reverse order as latter structures override earlier ones. To properly handle
+        # containment then we need to populate interval coordinates starting from the top.
+        # If a structure is found to be completely contained, the corresponding ``struct_bbox`` is
+        # set to ``None``.
+        for str_ind in range(len(structures) - 1, -1, -1):
+            # 3D and 2D bounding box of current structure
+            bbox = struct_bbox[str_ind]
+            if bbox is None:
+                # Structure has been removed because it is completely contained
+                continue
+            bbox_2d = IndexedBox(bbox[0, 0], bbox[0, 1], bbox[1, 0], bbox[1, 1])
 
-            # indexes of structures that the current structure contains in 2D
-            struct_contains.append(self.contains_2d(bbox, struct_bbox[:struct_ind]))
+            # List of structure indexes that may intersect the current structure in 2D
+            query_inds = [box.str_ind for box in tree.query(bbox_2d)]
 
-            # Figure out where to place the bounding box coordinates of current structure
-            indsmin = np.argwhere(bbox[0, 2] <= interval_coords)
-            indmin = int(indsmin[0])
-            bbox0 = np.copy(bbox)
-            bbox0[1, 2] = bbox0[0, 2]
-            if not self.is_contained(bbox0, struct_bbox[struct_ind + 1 :]):
-                # Add current structure bounding box coordinates
-                interval_coords = np.insert(interval_coords, indmin, bbox[0, 2])
-                # Copy the structure containment list to the newly created interval
-                struct_list = interval_structs[max(0, indmin - 1)]
-                interval_structs.insert(indmin, struct_list.copy())
+            # Remove all lower structures that the current structure completely contains
+            inds_lower = [ind for ind in query_inds if ind < str_ind]
+            query_bbox = [struct_bbox[ind] for ind in inds_lower if struct_bbox[ind] is not None]
+            bbox_contains_inds = self.contains_3d(bbox, query_bbox)
+            for ind in bbox_contains_inds:
+                struct_bbox[inds_lower[ind]] = None
 
-            indsmax = np.argwhere(bbox[1, 2] >= interval_coords)
-            indmax = int(indsmax[-1])
-            bbox0 = np.copy(bbox)
-            bbox0[0, 2] = bbox0[1, 2]
-            if not self.is_contained(bbox0, struct_bbox[struct_ind + 1 :]):
-                indmax += 1
-                # Add current structure bounding box coordinates
-                interval_coords = np.insert(interval_coords, indmax, bbox[1, 2])
-                # Copy the structure containment list to the newly created interval
-                struct_list = interval_structs[min(indmax - 1, len(interval_structs) - 1)]
-                interval_structs.insert(indmax, struct_list.copy())
+            # List of structure bboxes that contain the current structure in 2D
+            inds_upper = [ind for ind in query_inds if ind > str_ind]
+            query_bbox = [struct_bbox[ind] for ind in inds_upper if struct_bbox[ind] is not None]
+            bbox_contained_2d = self.contained_2d(bbox, query_bbox)
 
-            # Add the structure index to all intervals that it spans
-            for interval_ind in range(indmin, indmax):
-                interval_structs[interval_ind].append(struct_ind)
+            # Handle insertion of the current structure bounds in the intervals
+            intervals = self.insert_bbox(intervals, str_ind, bbox, bbox_contained_2d, min_step)
 
         # Truncate intervals to domain bounds
-        b_array = np.array(interval_coords)
-        in_domain = np.argwhere((b_array >= domain_bounds[0]) * (b_array <= domain_bounds[1]))
-        interval_coords = interval_coords[in_domain]
-        interval_structs = [interval_structs[int(i)] for i in in_domain if i < b_array.size - 1]
-
-        # Remove intervals that are smaller than the absolute smallest min_step
-        min_step = np.amin(structure_steps)
-        coords_filter = [interval_coords[0]]
-        structs_filter = []
-        for coord_ind, coord in enumerate(interval_coords[1:]):
-            if coord - coords_filter[-1] > min_step:
-                coords_filter.append(coord)
-                structs_filter.append(interval_structs[coord_ind])
-        interval_coords = np.array(coords_filter)
-        interval_structs = structs_filter
+        coords = np.array(intervals["coords"])
+        num_ints = len(intervals["structs"])
+        in_domain = np.argwhere((coords >= domain_bounds[0]) * (coords <= domain_bounds[1]))
+        intervals["coords"] = [intervals["coords"][int(i)] for i in in_domain]
+        intervals["structs"] = [intervals["structs"][int(i)] for i in in_domain if i < num_ints]
 
         # Compute the maximum allowed step size in each interval
         max_steps = []
-        for coord_ind, _ in enumerate(interval_coords[:-1]):
-            # Structure indexes inside current intervall in order of latter structures come first
-            struct_list = interval_structs[coord_ind]
-            struct_list_filter = []
-            # Handle containment
-            for ind, struct_ind in enumerate(struct_list[::-1]):
-                if ind >= len(struct_list):
-                    # This can happen because we modify struct_list in the loop
-                    break
-                # Add current structure to filtered list
-                struct_list_filter.append(struct_ind)
-                # Remove all structures that current structure overrides
-                contains = struct_contains[struct_ind]
-                struct_list = [ind for ind in struct_list if ind not in contains]
-
+        for coord_ind, _ in enumerate(intervals["coords"][:-1]):
             # Define the max step as the minimum over all medium steps of media in this interval
-            max_step = np.amin(structure_steps[struct_list_filter])
+            max_step = np.amin(structure_steps[intervals["structs"][coord_ind]])
             max_steps.append(float(max_step))
 
-        return interval_coords, np.array(max_steps)
+        # Re-evaluate the absolute smallest min_step and remove intervals that are smaller than that
+        intervals["coords"], max_steps = self.filter_min_step(intervals["coords"], max_steps)
+
+        return np.array(intervals["coords"]), np.array(max_steps)
+
+    # pylint:disable=too-many-locals,too-many-arguments
+    def insert_bbox(
+        self,
+        intervals: Dict[str, List],
+        str_ind: int,
+        str_bbox: Array[float],
+        bbox_contained_2d: List[Array[float]],
+        min_step: float,
+    ) -> Dict[str, List]:
+        """Figure out where to place the bounding box coordinates of current structure.
+        For both the left and the right bounds of the structure along the meshing direction,
+        we check if they are not too close to an already existing coordinate, and if the
+        structure is not completely covered by another structure at that location.
+        Only then we add that boundary to the list of interval coordinates.
+        We also don't add the bounds if ``str_ind==0``, since the domain bounds have already
+        been added to the interval coords at the start.
+        We also compute ``indmin`` and ``indmax`` indexes into the list of intervals, such that
+        the current structure is added to all intervals in the range (indmin, indmax).
+
+        Parameters
+        ----------
+        intervals : Dict[str, List]
+            Dictionary containing the coordinates of the interval boundaries, and a list
+            of lists of structures contained in each interval.
+        str_ind : int
+            Index of the current structure.
+        str_bbox : Array[float]
+            Bounding box of the current structure.
+        bbox_contained_2d : List[Array[float]]
+            List of 3D bounding boxes that contain the current structure in 2D.
+        min_step : float
+            Absolute minimum interval size to impose.
+        """
+
+        coords = intervals["coords"]
+        structs = intervals["structs"]
+
+        # Left structure bound
+        bound_coord = str_bbox[0, 2]
+        indsmin = np.argwhere(bound_coord <= coords)
+        indmin = int(indsmin[0])  # coordinate is in interval index ``indmin - 1````
+        is_close_l = self.is_close(bound_coord, coords, indmin - 1, min_step)
+        is_close_r = self.is_close(bound_coord, coords, indmin, min_step)
+        is_contained = self.is_contained(bound_coord, bbox_contained_2d)
+
+        # Decide on whether coordinate should be inserted or indmin modified
+        if is_close_l:
+            # Don't insert coordinate but decrease indmin
+            indmin -= 1
+        elif not is_close_r and not is_contained and str_ind > 0:
+            # Add current structure bounding box coordinates
+            coords.insert(indmin, bound_coord)
+            # Copy the structure containment list to the newly created interval
+            struct_list = structs[max(0, indmin - 1)]
+            structs.insert(indmin, struct_list.copy())
+
+        # Right structure bound
+        bound_coord = str_bbox[1, 2]
+        indsmax = np.argwhere(bound_coord >= coords)
+        indmax = int(indsmax[-1])  # coordinate is in interval index ``indmax``
+        is_close_l = self.is_close(bound_coord, coords, indmax, min_step)
+        is_close_r = self.is_close(bound_coord, coords, indmax + 1, min_step)
+        is_contained = self.is_contained(bound_coord, bbox_contained_2d)
+
+        # Decide on whether coordinate should be inserted or indmax modified
+        if is_close_r:
+            # Don't insert coordinate but increase indmax
+            indmax += 1
+        elif not is_close_l and not is_contained and str_ind > 0:
+            indmax += 1
+            # Add current structure bounding box coordinates
+            coords.insert(indmax, bound_coord)
+            # Copy the structure containment list to the newly created interval
+            struct_list = structs[min(indmax - 1, len(structs) - 1)]
+            structs.insert(indmax, struct_list.copy())
+
+        # Add the current structure index to all intervals that it spans, if it is not
+        # contained in any of the latter structures
+        for interval_ind in range(indmin, indmax):
+            # Check at the midpoint to avoid numerical issues at the interval boundaries
+            mid_coord = (coords[interval_ind] + coords[interval_ind + 1]) / 2
+            if not self.is_contained(mid_coord, bbox_contained_2d):
+                structs[interval_ind].append(str_ind)
+
+        return {"coords": coords, "structs": structs}
 
     @staticmethod
     def structure_steps(
@@ -225,68 +305,89 @@ class GradedMesher(Mesher):
         return struct_bbox
 
     @staticmethod
-    def is_contained(bbox0: Array[float], bbox_list: List[Array[float]]) -> bool:
-        """Check if a bounding box is contained in any of a list of bounding boxes.
+    def bounds_2d_tree(struct_bbox: List[Array[float]]):
+        """Make a shapely Rtree for the 2D bounding boxes of all structures in the plane
+        perpendicular to the meshing axis."""
 
-        Parameters
-        ----------
-        bbox0 : Array[float]
-            Bounding box to check.
-        bbox_list : List[Array[float]]
-            List of bounding boxes to check if they contain ``bbox0``.
-
-        Returns
-        -------
-        contained : bool
-            ``True`` if ``bbox0`` is contained in any of the boxes in the list.
-        """
-        contained = False
-        for bounds in bbox_list:
-            # It can be much faster to write out the conditions one by one than e.g. to use np.all
-            # on the bottom values and np.all on the top values
-            if all(
-                [
-                    bbox0[0, 0] >= bounds[0, 0],
-                    bbox0[1, 0] <= bounds[1, 0],
-                    bbox0[0, 1] >= bounds[0, 1],
-                    bbox0[1, 1] <= bounds[1, 1],
-                    bbox0[0, 2] >= bounds[0, 2],
-                    bbox0[1, 2] <= bounds[1, 2],
-                ]
-            ):
-                contained = True
-        return contained
+        boxes_2d = []
+        for str_ind, bbox in enumerate(struct_bbox):
+            box = IndexedBox(bbox[0, 0], bbox[0, 1], bbox[1, 0], bbox[1, 1])
+            box.str_ind = str_ind
+            boxes_2d.append(box)
+        return STRtree(boxes_2d)
 
     @staticmethod
-    def contains_2d(bbox0: Array[float], bbox_list: List[Array[float]]) -> List[int]:
-        """Check if a bounding box contains along the first two dimensions any of a list of
-        bounding boxes.
-
-        Parameters
-        ----------
-        bbox0 : Array[float]
-            Bounding box to check.
-        bbox_list : List[Array[float]]
-            List of bounding boxes to check if they are contained in ``bbox0``.
-
-        Returns
-        -------
-        List[int]
-            A list with all the indexes into the ``bbox_list`` that are contained in ``bbox0``
-            along the first two dimensions.
-        """
-        struct_contains_inds = []
-        for ind, bounds in enumerate(bbox_list):
+    def contained_2d(bbox0: Array[float], query_bbox: List[Array[float]]) -> List[Array[float]]:
+        """Return a list of all bounding boxes among ``query_bbox`` that contain ``bbox0`` in 2D."""
+        contained_in = []
+        for bbox in query_bbox:
             if all(
                 [
-                    bbox0[0, 0] <= bounds[0, 0],
-                    bbox0[1, 0] >= bounds[1, 0],
-                    bbox0[0, 1] <= bounds[0, 1],
-                    bbox0[1, 1] >= bounds[1, 1],
+                    bbox0[0, 0] >= bbox[0, 0],
+                    bbox0[1, 0] <= bbox[1, 0],
+                    bbox0[0, 1] >= bbox[0, 1],
+                    bbox0[1, 1] <= bbox[1, 1],
                 ]
             ):
-                struct_contains_inds.append(ind)
-        return struct_contains_inds
+                contained_in.append(bbox)
+        return contained_in
+
+    @staticmethod
+    def contains_3d(bbox0: Array[float], query_bbox: List[Array[float]]) -> List[int]:
+        """Return a list of all indexes of bounding boxes in the ``query_bbox`` list that ``bbox0``
+        fully contains."""
+        contains = []
+        for ind, bbox in enumerate(query_bbox):
+            if all(
+                [
+                    bbox[0, 0] >= bbox0[0, 0],
+                    bbox[1, 0] <= bbox0[1, 0],
+                    bbox[0, 1] >= bbox0[0, 1],
+                    bbox[1, 1] <= bbox0[1, 1],
+                    bbox[0, 2] >= bbox0[0, 2],
+                    bbox[1, 2] <= bbox0[1, 2],
+                ]
+            ):
+                contains.append(ind)
+        return contains
+
+    @staticmethod
+    def is_close(coord: float, interval_coords: List[float], coord_ind: int, atol: float) -> bool:
+        """Check if a given ``coord`` is within ``atol`` of an interval coordinate at a given
+        interval index. If the index is out of bounds, return ``False``."""
+        is_close = False
+        if 0 <= coord_ind < len(interval_coords):
+            is_close = isclose(coord, interval_coords[coord_ind], abs_tol=atol)
+        return is_close
+
+    @staticmethod
+    def is_contained(normal_pos: float, contained_2d: List[Array[float]]) -> bool:
+        """Check if a given ``normal_pos`` along the meshing direction is contained inside any
+        of the bounding boxes that are in the ``contained_2d`` list.
+        """
+        for contain_box in contained_2d:
+            if contain_box[0, 2] <= normal_pos <= contain_box[1, 2]:
+                return True
+        return False
+
+    @staticmethod
+    def filter_min_step(
+        interval_coords: List[float], max_steps: List[float]
+    ) -> Tuple[List[float], List[float]]:
+        """Filter intervals that are smaller than the absolute smallest of the ``max_steps``."""
+
+        # Re-compute minimum step in case some high-index structures were completely covered
+        min_step = np.amin(max_steps)
+
+        # Filter interval coordintaes and max_steps
+        coords_filter = [interval_coords[0]]
+        steps_filter = []
+        for coord_ind, coord in enumerate(interval_coords[1:]):
+            if coord - coords_filter[-1] > min_step:
+                coords_filter.append(coord)
+                steps_filter.append(max_steps[coord_ind])
+
+        return coords_filter, steps_filter
 
     def make_grid_multiple_intervals(  # pylint:disable=too-many-locals
         self,
