@@ -1,7 +1,8 @@
 # pylint: disable=too-many-lines, too-many-arguments
 """ Container holding all information about simulation and its components"""
 from typing import Dict, Tuple, List, Set, Union
-from functools import lru_cache
+from math import isclose
+from functools import wraps
 
 import pydantic
 import numpy as np
@@ -17,7 +18,7 @@ from .types import Symmetry, Ax, Shapely, FreqBound, Axis, GridSize
 from .grid import Coords1D, Grid, Coords, GridSpec, UniformGrid
 from .medium import Medium, MediumType, AbstractMedium, PECMedium
 from .structure import Structure
-from .source import SourceType, PlaneWave
+from .source import SourceType, PlaneWave, GaussianBeam, AstigmaticGaussianBeam
 from .monitor import MonitorType, Monitor, FreqMonitor
 from .pml import PMLTypes, PML, Absorber
 from .viz import add_ax_if_none, equal_aspect
@@ -27,7 +28,7 @@ from .viz import plot_params_structure, plot_params_pml, plot_params_override_st
 from ..version import __version__
 from ..constants import C_0, MICROMETER, SECOND, inf
 from ..log import log, Tidy3dKeyError, SetupError, ValidationError
-
+from ..static import make_static
 
 # minimum number of grid points allowed per central wavelength in a medium
 MIN_GRIDS_PER_WVL = 6.0
@@ -42,6 +43,29 @@ MAX_CELLS_TIMES_STEPS = 1e17
 MAX_MONITOR_DATA_SIZE_BYTES = 10e9
 
 
+def cache(prop):
+    """Decorates a property to cache the previously computed values based on a hash of self."""
+
+    stored_values = {}
+
+    @wraps(prop)
+    def cached_property(self):
+        """The new property method to be returned by decorator."""
+
+        hash_key = hash(self)
+
+        # if the value has been computed before, we can simply return the stored value
+        if hash_key in stored_values:
+            return stored_values[hash_key]
+
+        # otherwise, we need to recompute the value, store it in the storage dict, and return
+        return_value = prop(self)
+        stored_values[hash_key] = return_value
+        return return_value
+
+    return cached_property
+
+
 class Simulation(Box):  # pylint:disable=too-many-public-methods
     """Contains all information about Tidy3d simulation.
 
@@ -192,6 +216,29 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         description="String specifying the front end version number.",
     )
 
+    """ Static simulation tools."""
+
+    _hash: int = pydantic.PrivateAttr(None)
+
+    def __hash__(self) -> int:
+        """Hash a :class:`Tidy3dBaseModel` objects using its json string."""
+        if self._hash is not None:
+            return self._hash
+
+        return super().__hash__()
+
+    def _freeze(self) -> int:
+        """Computes the hash, sets ``self._hash``, and returns it."""
+        frozen_hash = hash(self)
+        self._hash = frozen_hash
+        return frozen_hash
+
+    def _unfreeze(self) -> int:
+        """Sets ``self._hash`` to ``None``, returns final hash."""
+        self._hash = None
+        final_hash = hash(self)
+        return final_hash
+
     """ Validating setup """
 
     @pydantic.validator("grid_size", always=True)
@@ -274,7 +321,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
             for sim_val, struct_val in zip(sim_bounds, struct_bounds):
 
-                if np.isclose(sim_val, struct_val):
+                if isclose(sim_val, struct_val):
                     log.warning(
                         f"Structure at structures[{istruct}] has bounds that extend exactly to "
                         "simulation edges. This can cause unexpected behavior. "
@@ -448,8 +495,8 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         return val
 
     @pydantic.validator("sources", always=True)
-    def _plane_wave_homogeneous(cls, val, values):
-        """Error if plane wave intersects"""
+    def _source_homogeneous(cls, val, values):
+        """Error if a plane wave or gaussian beam source is not in a homogeneous region."""
 
         if val is None:
             return val
@@ -469,7 +516,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
         # for each plane wave in the sources list
         for source in val:
-            if isinstance(source, PlaneWave):
+            if isinstance(source, (PlaneWave, GaussianBeam, AstigmaticGaussianBeam)):
 
                 # get all merged structures on the plane
                 normal_axis_index = source.size.index(0.0)
@@ -483,7 +530,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
                 if len(mediums) > 1:
                     raise SetupError(
                         f"{len(mediums)} different mediums detected on plane "
-                        "intersecting a plane wave source.  Plane must be homogeneous."
+                        f"intersecting a {source.type} source. Plane must be homogeneous."
                     )
 
         return val
@@ -552,7 +599,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
     """ Accounting """
 
     @property
-    @lru_cache()
+    @cache
     def mediums(self) -> Set[MediumType]:
         """Returns set of distinct :class:`AbstractMedium` in simulation.
 
@@ -566,7 +613,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         return list(medium_dict.keys())
 
     @property
-    @lru_cache()
+    @cache
     def medium_map(self) -> Dict[MediumType, pydantic.NonNegativeInt]:
         """Returns dict mapping medium to index in material.
         ``medium_map[medium]`` returns unique global index of :class:`AbstractMedium` in simulation.
@@ -596,6 +643,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
     @equal_aspect
     @add_ax_if_none
+    @make_static
     def plot(
         self,
         x: float = None,
@@ -639,6 +687,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
     @equal_aspect
     @add_ax_if_none
+    @make_static
     def plot_eps(  # pylint:disable=too-many-arguments
         self,
         x: float = None,
@@ -1382,7 +1431,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         return len(self.tmesh)
 
     @property
-    @lru_cache()
+    @cache
     def grid(self) -> Grid:
         """FDTD grid spatial locations and information.
 
@@ -1543,9 +1592,14 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         def make_eps_data(coords: Coords):
             """returns epsilon data on grid of points defined by coords"""
             xs, ys, zs = coords.x, coords.y, coords.z
+            rmin = tuple(coord[0] for coord in (xs, ys, zs))
+            rmax = tuple(coord[-1] for coord in (xs, ys, zs))
+            points_box = Box.from_bounds(rmin=rmin, rmax=rmax)
             x, y, z = np.meshgrid(xs, ys, zs, indexing="ij")
             eps_array = eps_background * np.ones(x.shape, dtype=complex)
             for structure in self.structures:
+                if not points_box.intersects(structure.geometry):
+                    continue
                 eps_structure = get_eps(structure.medium, freq)
                 is_inside = structure.geometry.inside(x, y, z)
                 eps_array[np.where(is_inside)] = eps_structure
